﻿using ChromeStarter.Models;
using Microsoft.Win32;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace ChromeStarter
{
    /// <summary>
    /// MainWindow.xaml에 대한 상호 작용 논리
    /// </summary>
    public partial class MainWindow : Window
    {
        private readonly string PROFILE_PATTERN = "Profile ";
        private readonly string CHROME_PATH_KEY = "ChromePath";

        private string _chromePath;
        string _savePath;
        private List<string> _userProfileNames;
        private PlanCollection _profilePlanCollection;

        public MainWindow()
        {
            InitializeComponent();
            InitInstance();
        }

        private void InitInstance()
        {
            _chromePath = ConfigurationManager.AppSettings[CHROME_PATH_KEY];
            _savePath = $@"{AppDomain.CurrentDomain.BaseDirectory}\cache.json";
            _profilePlanCollection = this.DataContext as PlanCollection;

            if (!string.IsNullOrEmpty(_chromePath) )
                tbChromePath.Text = _chromePath;

            if (_profilePlanCollection == null)
            {
                ShowErrorMessage("윈도우 컨텍스트를 확인할 수 없습니다.");
                return;
            }

            string localChromeUserDataPath = $@"{Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData)}\Google\Chrome\User Data";
            if (!Directory.Exists(localChromeUserDataPath))
            {
                ShowErrorMessage("사용자 로컬 폴더를 찾을 수 없습니다. 크롬이 설치되어있는지 확인하십시오.");
                return;
            }

            _userProfileNames = GetProfiles(localChromeUserDataPath);
            lbProfileLabel.Content = $"Target Profile: {_userProfileNames.Count}";
            if (_userProfileNames == null || _userProfileNames.Count < 1)
            {
                ShowErrorMessage("사용자 프로필을 확인할 수 없습니다. 크롬에서 사용자 프로필을 생성하십시오.");
                return;
            }

            List<string> cache = new List<string>();
            if (File.Exists(_savePath))
            {
                string jsonString = File.ReadAllText(_savePath);
                cache = JsonConvert.DeserializeObject<List<string>>(jsonString);
            }

            foreach (string startPage in cache)
            {
                _profilePlanCollection.Plans.Add(new Plan() { StartPage = startPage });
            }
        }

        private List<string> GetProfiles(string localChromeUserDataPath)
        {
            string[] subDirectories = Directory.GetDirectories(localChromeUserDataPath);
            var filteredDirectories = subDirectories.Where(dir => System.IO.Path.GetFileName(dir).StartsWith(PROFILE_PATTERN));
            if (!filteredDirectories.Any())
                return null;

            List<string> profiles = new List<string>();
            foreach (string dir in filteredDirectories)
            {
                profiles.Add(System.IO.Path.GetFileName(dir));
            }

            return profiles;
        }

        private void ShowErrorMessage(string message)
        {
            MessageBox.Show(message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
        }

        private void SavePlanCache(PlanCollection planCollection)
        {
            List<string> cache = new List<string>();
            foreach (var profilePlan in planCollection.Plans)
            {
                cache.Add(profilePlan.StartPage);
            }

            string jsonString = JsonConvert.SerializeObject(cache);

           
            File.WriteAllText(_savePath, jsonString);

        }

        private void btnFindChromePath_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            if (dlg.ShowDialog() != true)
                return;

            tbChromePath.Text = dlg.FileName;
            Configuration config = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
            config.AppSettings.Settings[CHROME_PATH_KEY].Value = dlg.FileName;

            config.Save(ConfigurationSaveMode.Modified);
        }

        private void btnStart_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (string.IsNullOrEmpty(tbChromePath.Text))
                {
                    ShowErrorMessage("크롬 실행파일 위치를 설정해야합니다.");
                    return;
                }

                if (_profilePlanCollection == null)
                {
                    ShowErrorMessage("실행 계획을 생성할 수 없습니다.");
                    return;
                }

                List<Plan> removeTarget = new List<Plan>();
                foreach (Plan plan in _profilePlanCollection.Plans)
                {
                    if (string.IsNullOrEmpty(plan.StartPage))
                        removeTarget.Add(plan);
                }

                foreach (Plan plan in removeTarget)
                {
                    _profilePlanCollection.Plans.Remove(plan);
                }

                List<string> parameters = _profilePlanCollection.GetCommandParameter(_userProfileNames);
                if (parameters.Count < 1)
                {
                    ShowErrorMessage("실행 계획 파라미터를 생성할 수 없습니다.");
                    return;
                }
                foreach (string profilePlan in parameters)
                {
                    Process.Start(tbChromePath.Text, profilePlan);
                }

                SavePlanCache(_profilePlanCollection);
            }
            catch (Exception ex)
            {
                ShowErrorMessage(ex.Message);
            }
        }

        private void btnAddStartpage_Click(object sender, RoutedEventArgs e)
        {
            _profilePlanCollection.Plans.Add(new Plan() { StartPage = "" });
        }

        private void btnRemoveStartpage_Click(object sender, RoutedEventArgs e)
        {
            Plan lastPlan = _profilePlanCollection.Plans.LastOrDefault();
            if (lastPlan != null)
                _profilePlanCollection.Plans.Remove(lastPlan);
        }
    }
}
